#!/ffp/bin/sh

# PROVIDE: minidlna

########################################################################################################################
#MiniDLNA (aka ReadyDLNA) is server software with the aim of being fully compliant with DLNA/UPnP-AV clients.          #
#MiniDLNA dependencies required:ffmpeg with it's own dependencies, libjpeg, flac, libexif, libid3tag, libogg, libvorbis#
#sqlite, gettext, intltool                                                                                             #
#                                                                                                                      #
#http://sourceforge.net/projects/minidlna/                                                                             #
#                                                                                                                      #
#MiniDLNA daemon startup-shutdown-control script for FFP by barmalej2                                                  #
########################################################################################################################
# Logging is enabled by default. Logs are located in /ffp/var/log/minidlna.log file.
# Verbosity  of logs is controlled in minidlna config_file-/ffp/etc/minidlna.conf by parameter-log_level.
# Default action is adjusted to log errors only.

# WARNING!!! Stop minidlna service first, before editing variables in this script!!!
# Otherwise status, start, stop, restart, reload and rescan commands of this script will fail.

# Make sure the configuration file exists. If yes, import settings.
if	[ -f /ffp/etc/minidlna.conf ]
then
	. /ffp/etc/minidlna.conf
else
	echo -e "\e[1;31;20mERROR WARNING: MiniDLNA config file /ffp/etc/minidlna.conf was not found. Exiting...\e[0m"
	exit 1
fi

name="minidlna"
command="/ffp/bin/$name"
home_dir="$db_dir"
pid_dir="/ffp/var/run/$name/"
config_file="/ffp/etc/$name.conf"
required_files="$command"
runas_user="$user"
minidlna_args="-f $config_file"

if	[ -n "$minidlna_args" ]
then
	command="$command $minidlna_args"
fi

# Check required_files
for req_file in $required_files
do
	if	[ ! -r "$req_file" ]
	then
		echo -e "\e[1;31;20m$req_file: Required file not found or not readable!\e[0m"
		exit 1
	fi
done

add_user()
{
	grep "^$runas_user:" /etc/group &>/dev/null || /ffp/sbin/groupadd -r "$runas_user" >/dev/null
	/ffp/bin/id -u "$runas_user" &>/dev/null || /ffp/sbin/useradd -r -g "$runas_user" -d "$home_dir" -s /ffp/bin/false -c MiniDLNA "$runas_user" >/dev/null
}

check_home()
{
	# Create dirs if necessary 
	# If $home_dir or $pid_dir was owned by the different user from current, we need to own them
	for dir in "$home_dir" "$pid_dir"
	do
		if	[ ! -d "$dir" ]
		then
			mkdir -p "$dir" || { echo -e "\e[1;31;20mFail!!! ERROR WARNING: Can't create $dir directory-aborting start of $name due to this error!!!\e[0m"; exit 1; }
			chown -R "$runas_user":"$runas_user" "$dir"
		else
			if	[ "$(stat -c '%U' "$dir")" != "$runas_user" ]
			then
				chown -R "$runas_user":"$runas_user" "$dir"
			else
				find "$dir" ! -user "$runas_user" -exec chown "$runas_user":"$runas_user" {} \;
			fi
		fi
	done
	# Increase max_user_watches
	echo "66000" > /proc/sys/fs/inotify/max_user_watches
}

check_status()
{
	status="$(pgrep -f "$command")"
	retvalue="$?"
}

minidlna_start()
{
	check_status
	if	[ "$retvalue" -eq "1" ]
	then
		add_user
		check_home
		echo -n "Starting $name..."
		$command
		sleep 1
		check_status
		if	[ "$retvalue" -eq "0" ]
		then
			echo -e "done! \e[1;32;20m[ OK ]\e[0m"
		else
			echo -e "\e[1;31;20m[ Fail!!! ] ERROR WARNING: $name can not start!!!\e[0m"
			exit 1
		fi
	else
		if	[ "$retvalue" -eq "0" ]
		then
			echo -e "PROCESS STATUS INFO: \e[1;33;20m$name is already running with process ID(s):${status[*]}\e[0m"
			ps -fj -C "$name"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-start command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_stop()
{
	check_status
	if	[ "$retvalue" -eq "0" ]
	then
		echo -n "Stopping $name..."
		pkill -fP1 "$command"
		local timeout=0
		while [ "$timeout" -lt 5 ]
		do
			sleep 1
			check_status
			if	[ "$retvalue" -eq "1" ]
			then
				echo -e "done! \e[1;32;20m[ OK ]\e[0m"
				return 0
			fi
			let timeout=timeout+1
		done
		if	[ "$retvalue" -eq "0" ]
		then
			echo -e "\e[1;31;20m[ Fail!!! ] ERROR WARNING: $name failed to exit cleanly within $timeout seconds. Timeout reached!\e[0m"
			echo -n "Stopping $name by force..."
			pkill -9 -f "$command"
			local timeout=0
			while [ "$timeout" -lt 2 ]
			do
				check_status
				if	[ "$retvalue" -eq "1" ]
				then
					echo -e "done! \e[1;32;20m[ OK ]\e[0m"
					return 0
				fi
				sleep 1
				let timeout=timeout+1
			done
			echo -e "\e[1;31;20m[ Fail!!! ] ERROR WARNING: $name can not be stopped!!!\e[0m"
			exit 1
		else
			echo -e "\e[1;31;20m[ Fail!!! ]ERROR WARNING: Can not determine running status of $name-stopping command failed!!!\e[0m"
			exit 1
		fi
	else
		if	[ "$retvalue" -eq "1" ]
		then
			echo -e "PROCESS STATUS INFO: \e[1;33;20m$name is not running\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-stopping command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_status()
{
	check_status
	echo -n "PROCESS STATUS INFO: "
	if	[ "$retvalue" -eq "0" ]
	then
		echo -e "\e[1;33;20m$name is running with process ID(s):${status[*]}\e[0m"
		ps -fj -C "$name"
	else
		if	[ "$retvalue" -eq "1" ]
		then
			echo -e "\e[1;33;20m$name is not running\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-status command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_reload()
{
	# This command will reload minidlna config file
	check_status
	if	[ "$retvalue" -eq "0" ]
	then
		echo -n "Reloading $name config file: $config_file..."
		if	pkill -HUP -fP1 "$command"
		then
			echo -e "done! \e[1;32;20m[ OK ]\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING:[ FAIL ]!!!\e[0m"
		fi
	else
		if	[ "$retvalue" -eq "1" ]
		then
			echo -e "PROCESS STATUS INFO: \e[1;33;20m$name is not running\e[0m"
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-reload command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_rebuild()
{
	check_status
	if	[ "$retvalue" -eq "1" ]
	then
		minidlna_args="$minidlna_args -R"
		command="/ffp/bin/$name $minidlna_args"
		minidlna_start && { echo -e "\e[1;33;20mFull MiniDLNA database rescan is in progress...\e[0m"; echo -e "\e[1;33;20mThis process can take for a while, depending on size and number of files used in media library!\e[0m"; }
	else
		if	[ "$retvalue" -eq "0" ]
		then
			minidlna_stop
			minidlna_args="$minidlna_args -R"
			command="/ffp/bin/$name $minidlna_args"
			minidlna_start && { echo -e "\e[1;33;20mFull MiniDLNA database rescan is in progress...\e[0m"; echo -e "\e[1;33;20mThis process can take for a while, depending on size and number of files used in media library!\e[0m"; }
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-rescan command failed!!!\e[0m"
			exit 1
		fi
	fi
}

minidlna_rescan()
{
	check_status
	if	[ "$retvalue" -eq "1" ]
	then
		minidlna_args="$minidlna_args -r"
		command="/ffp/bin/$name $minidlna_args"
		minidlna_start && { echo -e "\e[1;33;20mSoft MiniDLNA database rescan is in progress...\e[0m"; echo -e "\e[1;33;20mThis process can take for a while, depending on size and number of files used in media library!\e[0m"; }
	else
		if	[ "$retvalue" -eq "0" ]
		then
			minidlna_stop
			minidlna_args="$minidlna_args -r"
			command="/ffp/bin/$name $minidlna_args"
			minidlna_start && { echo -e "\e[1;33;20mSoft MiniDLNA database rescan is in progress...\e[0m"; echo -e "\e[1;33;20mThis process can take for a while, depending on size and number of files used in media library!\e[0m"; }
		else
			echo -e "\e[1;31;20mERROR WARNING: Can not determine running status of $name-rescan command failed!!!\e[0m"
			exit 1
		fi
	fi
}

case "$1" in
		start)
				minidlna_start
				;;
		stop)
				minidlna_stop
				;;
		restart)
				minidlna_stop
				sleep 1
				minidlna_start
				;;
		status)
				minidlna_status
				;;
		rebuild)
				minidlna_rebuild
				;;
		reload)
				minidlna_reload
				;;
		rescan)
				minidlna_rescan
				;;
		*)
				cat <<- EOM
				$(echo -e "\e[1;32;20mUsage: $0 (start|stop|restart|status|rebuild|reload|rescan)\e[0m")
				Use reload to update changed settings in config file for running minidlna service.
				Use rebuild command for full database rebuild from the scratch.
				Use rescan for non-destructive, soft database update.
				EOM
				exit 1
				;;
esac
